/*****************************************************************************
*
*	File Name:  Serial.C	                                         
*                                                                  
*	Content:    Part of Serial I/O support library
*				Provide direct reads and writes to RS232 serial ports
*				without using stdin and stdout.                  
*                                                                  
*	Copyright (c) 2002 Mitsubishi Electric & Electronics USA, Inc.                       
*	All rights reserved
*
*	Mitsubishi Electric & Electronics USA does not guarantee the performance or 
*	use of this source-code.  The intended use of provided source-code is the 
*	sole responsible of the user.  The files have been successfully compiled 
*	using Mitsubishi's NC30 compiler.  Before using this software review the 
*   source and make any necessary changes to support your hardware and application.
*                                                               
*=============================================================================
*	$Log:$
*===========================================================================*/
#define SERIAL26_C
#include "serial26.h"
void init_uart(void);			// Intial Serial UART0 of M16C/26 MCU. 
int char_recv(void);			// Test for presence of char in receive queue.
#pragma INTERRUPT Rcev_intr	
void   Rcev_intr(void);
#pragma INTERRUPT Xmit_intr
void Xmit_intr(void);
void xmit_schar(char c);		// Transmit single char to serial port/queue. 
char recv_schar(void);			// Receive single char from receive serial queue.

//Private Declarations *****************************************************
#pragma EQU _u0mr  	=   0x3A0           /* UART0 transfer&receive mode reg. U0MR  */
#pragma EQU _u0brg  =   0x3A1           /*       Baud Rate Register.        U0BRG */
#pragma EQU _u0tb  	=   0x3A2           /*       transfer buffer reg.       U0TB  */
#pragma EQU _u0c0 	=  	0x3A4           /*       control reg(L).            U0C0  */
#pragma EQU _u0c1 	=  	0x3A5           /*       control reg(H).            U0C1  */
#pragma EQU _u0rb   =   0x3A6           /*       receive buffer reg.        U0RB  */
#pragma EQU _s0tic  = 	0x051			/* 	     Xmit interrupt Control reg.S0TIC */
#pragma EQU _s0ric 	= 	0x052			/*       Rcev interrupt Control reg.S0RIC */
#pragma EQU _ucon =     0x3B0           /*       UART transmit/receive control register 2 UCON */

char _u0mr;
char _u0brg;
char _u0tb;
char _u0c0;
char _u0c1;
char _u0rb;
char _s0tic;
char _s0ric;
char _ucon;

// Some handy MACROS **************************************************************
#define TXEPTY  0x8                     /* transfer register empty */
#define TE      0x1                     /* transfer enable         */
#define TI      0x2                     /* transfer buffer empty   */
#define RE      0x4                     /* receive enable          */
#define RI      0x8                     /* receive complete        */

#define ACK     0x40
#define BUSY    0x20
#define FAULT   0x10
#define RESET   0x8
#define STROBE  0x4

#ifndef FALSE
#define FALSE	0
#endif
#ifndef TRUE
#define TRUE    1
#endif
/********************************************************************************/

// ********* QUEUE STRUCTURES ************ //
char Ser_XMIT_q[XMIT_BUFFER_SIZE];
int SXMIT_IN, SXMIT_OUT;

char Ser_RCEV_q[RECEIVE_BUFFER_SIZE];
int SRECV_IN, SRECV_OUT;
// ************************************** //

/*****************************************************************************
Name:          init_uart 
Parameters:                     
Returns:        
Description:   Initializes UART0 for RS232 serial communications.  

*****************************************************************************/
void init_uart(void)
{
	int trash;
	_ucon = 0x40;			   // Set RXD0 input port to p6_2.
	_u0brg = BRG_REG_VALUE;
	_u0mr = 0x05;			   // 8 bits, Internal clock, one stop bit, Parity disabled, 
							   // sleep mode deselected.
	_u0c0 = DIV_RATE;		   // Div_rate, TXD is CMOS output.

// Initialize Queues. **********************************************************
	SXMIT_IN = 	0;			   // Init queues indexes.  
	SXMIT_OUT = 0;
 	SRECV_IN = 	0;
	SRECV_OUT = 0;

// Initialize RECV Interrupts. ******************************************************  	
   	_u0c1 |= RE;    			// receive enable
	while(_u0c1 & 0x08)  		// Clear receive buffer.
		trash = _u0rb;
	_u0c1 &= 0x0b;    			// receive disable

	asm("FCLR I");			 	// disable interrupts
	_s0ric = 0x02;		 		// enable receive interrupt
   	_u0c1 |= RE;    			// receive enable
	asm("FSET I");	     		// enable interrupts
}

/*****************************************************************************
Name:            char_recv
Parameters:                     
Returns:         1 if received data is in the receive queue.
Description:     Checks queue buffer for receive data. 
*****************************************************************************/
int char_recv(void)
{
	if(SRECV_OUT == SRECV_IN)
		return 0;
	else
		return 1;
}	

/*****************************************************************************
Name:          Xmit_intr 
Parameters:                     
Returns:        
Description:    Interrupt handler for the transmit buffer. 
*****************************************************************************/
void   Xmit_intr(void)
{
	if(SXMIT_OUT != SXMIT_IN){ 		// more characters to transmit.
	 	_u0tb = Ser_XMIT_q[SXMIT_OUT];
	 	if(++SXMIT_OUT >= XMIT_BUFFER_SIZE)
			SXMIT_OUT = 0;	
	 	}
	else {	 						// transmit buffer empty
		_u0c1 &= 0x04;           	// transfer disable
   		asm("FCLR I");			 	// disable interrupts
	 	_s0tic = 0x00;				// disable transmit interrupt
	    asm("FSET I");				// enable interrupts
		}
}	

/*****************************************************************************
Name:          Rcev_intr 
Parameters:                     
Returns:        
Description:   Interrupt handler for the receive buffer  
*****************************************************************************/
void   Rcev_intr(void)
{
	static	char received;
	received = _u0rb;	
	Ser_RCEV_q[SRECV_IN] = 	received;
	if(++SRECV_IN >= RECEIVE_BUFFER_SIZE)
		 SRECV_IN = 0;
   	_u0c1 |= RE;    				// receive enable
}


/*****************************************************************************
Name:           xmit_schar
Parameters:     (char c)                
Returns:        
Description:    Transmit single char to serial port/queue.

*****************************************************************************/
void xmit_schar(char c)		// transmit single char to serial buff  
{
	Ser_XMIT_q[SXMIT_IN] = c;
	if(++SXMIT_IN >= XMIT_BUFFER_SIZE)
		SXMIT_IN = 0;	 
	if(!(_u0c1 & TE)){					   	// Not TE enabled. 
	   _u0c1 |= 0x01;            			// transfer enable
   		asm("FCLR I");			 			// disable interrupts
	 	_s0tic = 0x01;
	    asm("FSET I");	     				// enable interrupts
	 	_u0tb = Ser_XMIT_q[SXMIT_OUT];
	 	if(++SXMIT_OUT >= XMIT_BUFFER_SIZE)
			SXMIT_OUT = 0;	
		}
wait_for_tranmission:
 	if(SXMIT_IN >= SXMIT_OUT){				// Check for full queue.
 		if ((SXMIT_IN - SXMIT_OUT) == (XMIT_BUFFER_SIZE - 1))
			goto wait_for_tranmission;
 		}
 	else {// SXMIT_IN < SXMIT_OUT
 		if((SXMIT_OUT - SXMIT_IN) <= 1)		// Buffer full 
 			goto wait_for_tranmission; 
 		}
}

/*****************************************************************************
Name:           recv_schar
Parameters:                     
Returns:        char
Description:    Receive single char from receive serial queue.
*****************************************************************************/
char recv_schar(void)			// Receive single char from receive serial queue.
{
    char  data;
	if(SRECV_OUT != SRECV_IN){
    	data = Ser_RCEV_q[SRECV_OUT];
		if(++SRECV_OUT >= RECEIVE_BUFFER_SIZE)
			SRECV_OUT = 0; 
    	return data;
		}
	else
		return 0xff;
}
